@@ -66,7 +66,13 @@ using namespace lt;
 #include <conio.h>
 #endif
 
-std::shared_ptr<torrent_info> generate_torrent(bool const with_files)
+#if defined TORRENT_WINDOWS
+#define SEPARATOR "\\"
+#else
+#define SEPARATOR "/"
+#endif
+
+std::shared_ptr<torrent_info> generate_torrent(bool const with_files, bool const with_hashes)
 {
 	if (with_files)
 	{
@@ -90,11 +96,19 @@ std::shared_ptr<torrent_info> generate_torrent(bool const with_files)
 	t.add_url_seed("http://torrent_file_url_seed.com/");
 
 	TEST_CHECK(t.num_pieces() > 0);
-	for (auto const i : fs.piece_range())
+	if (with_hashes)
 	{
-		sha1_hash ph;
-		aux::random_bytes(ph);
-		t.set_hash(i, ph);
+		lt::set_piece_hashes(t, "."
+			, [] (lt::piece_index_t) {});
+	}
+	else
+	{
+		for (auto const i : fs.piece_range())
+		{
+			sha1_hash ph;
+			aux::random_bytes(ph);
+			t.set_hash(i, ph);
+		}
 	}
 
 	std::vector<char> buf;
@@ -380,7 +394,7 @@ void wait_for_downloading(lt::session& ses, char const* name)
 			}, false);
 		if (downloading_done) break;
 		if (total_seconds(clock_type::now() - start) > 10) break;
-		a = ses.wait_for_alert(seconds(2));
+		a = ses.wait_for_alert(seconds(5));
 	} while (a);
 	if (!downloading_done)
 	{
@@ -390,6 +404,31 @@ void wait_for_downloading(lt::session& ses, char const* name)
 	}
 }
 
+void wait_for_seeding(lt::session& ses, char const* name)
+{
+	time_point start = clock_type::now();
+	bool seeding = false;
+	alert const* a = nullptr;
+	do
+	{
+		seeding = print_alerts(ses, name, true, true
+			, [](lt::alert const* al)
+			{
+				state_changed_alert const* sc = alert_cast<state_changed_alert>(al);
+				return sc && sc->state == torrent_status::seeding;
+			}, false);
+		if (seeding) break;
+		if (total_seconds(clock_type::now() - start) > 10) break;
+		a = ses.wait_for_alert(seconds(5));
+	} while (a);
+	if (!seeding)
+	{
+		std::printf("%s: did not receive a state_changed_alert indicating "
+			"the torrent is seeding. waited: %d ms\n"
+			, name, int(total_milliseconds(clock_type::now() - start)));
+	}
+}
+
 void print_ses_rate(float const time
 	, lt::torrent_status const* st1
 	, lt::torrent_status const* st2
@@ -644,25 +683,25 @@ int start_proxy(int proxy_type)
 		case settings_pack::socks4:
 			type = "socks4";
 			auth = " --allow-v4";
-			cmd = "../socks.py";
+			cmd = ".." SEPARATOR "socks.py";
 			break;
 		case settings_pack::socks5:
 			type = "socks5";
-			cmd = "../socks.py";
+			cmd = ".." SEPARATOR "socks.py";
 			break;
 		case settings_pack::socks5_pw:
 			type = "socks5";
 			auth = " --username testuser --password testpass";
-			cmd = "../socks.py";
+			cmd = ".." SEPARATOR "socks.py";
 			break;
 		case settings_pack::http:
 			type = "http";
-			cmd = "../http_proxy.py";
+			cmd = ".." SEPARATOR "http_proxy.py";
 			break;
 		case settings_pack::http_pw:
 			type = "http";
 			auth = " --basic-auth testuser:testpass";
-			cmd = "../http_proxy.py";
+			cmd = ".." SEPARATOR "http_proxy.py";
 			break;
 	}
 	std::string python_exe = get_python();
@@ -683,12 +722,6 @@ int start_proxy(int proxy_type)
 
 using namespace lt;
 
-template <class T>
-std::shared_ptr<T> clone_ptr(std::shared_ptr<T> const& ptr)
-{
-	return std::make_shared<T>(*ptr);
-}
-
 std::vector<char> generate_piece(piece_index_t const idx, int const piece_size)
 {
 	using namespace lt;
@@ -916,7 +949,7 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 	param.flags &= ~torrent_flags::paused;
 	param.flags &= ~torrent_flags::auto_managed;
 	if (p) param = *p;
-	param.ti = clone_ptr(t);
+	param.ti = t;
 	param.save_path = "tmp1" + suffix;
 	param.flags |= torrent_flags::seed_mode;
 	error_code ec;
@@ -941,7 +974,7 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 
 	if (ses3)
 	{
-		param.ti = clone_ptr(t);
+		param.ti = t;
 		param.save_path = "tmp3" + suffix;
 		tor3 = ses3->add_torrent(param, ec);
 		TEST_CHECK(!ses3->get_torrents().empty());
@@ -954,11 +987,11 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 	}
 	else if (torrent2)
 	{
-		param.ti = clone_ptr(*torrent2);
+		param.ti = *torrent2;
 	}
 	else
 	{
-		param.ti = clone_ptr(t);
+		param.ti = t;
 	}
 	param.save_path = "tmp2" + suffix;
 
@@ -1046,7 +1079,7 @@ int start_web_server(bool ssl, bool chunked_encoding, bool keepalive, int min_in
 	std::string python_exe = get_python();
 
 	char buf[200];
-	std::snprintf(buf, sizeof(buf), "%s ../web_server.py %d %d %d %d %d"
+	std::snprintf(buf, sizeof(buf), "%s .." SEPARATOR "web_server.py %d %d %d %d %d"
 		, python_exe.c_str(), port, chunked_encoding, ssl, keepalive, min_interval);
 
 	std::printf("%s starting web_server on port %d...\n", time_now_string(), port);
